// -*- C++ -*-
/*!
 * @file  VirtualH8.cpp
 * @brief VirtualH8
 * $Date$
 *
 * $Id$
 */

#include "VirtualH8Dummy.h"

// Module specification
// <rtc-template block="module_spec">
static const char* virtualh8_spec[] =
  {
    "implementation_id", "VirtualH8Dummy",
    "type_name",         "VirtualH8Dummy",
    "description",       "VirtualH8Dummy",
    "version",           "1.0",
    "vendor",            "Takasi Kojima,klab",
    "category",          "CommunicationH8",
    "activity_type",     "",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.H8_ip_address",DEFAULT_IP_ADDRESS,
    "conf.default.Port_number",DEFAULT_PORT,
    // Configuration variables

    ""
  };
// </rtc-template>

VirtualH8::VirtualH8(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_informUpdateInPortIn("informUpdateInPort", m_informUpdateInPort),
    m_informUpdateOutPortOut("informUpdateOutPort", m_informUpdateOutPort),
    m_requestIDPort("requestID"),
    // </rtc-template>
		m_IPAddress(DEFAULT_IP_ADDRESS),
		m_portNumber(atoi(DEFAULT_PORT)),
		processCounter(0),
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("informUpdateInPort", m_informUpdateInPortIn);
  
  // Set OutPort buffer
  registerOutPort("informUpdateOutPort", m_informUpdateOutPortOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_requestIDPort.registerConsumer("requestID", "requestID", m_requestID);
  
  // Set CORBA Service Ports
  registerPort(m_requestIDPort);
  
  // </rtc-template>

}

VirtualH8::~VirtualH8()
{
}


RTC::ReturnCode_t VirtualH8::onInitialize()
{
  // <rtc-template block="bind_config">
    // Bind variables and configuration variable
  bindParameter("H8_ip_address",m_IPAddress ,DEFAULT_IP_ADDRESS);
  bindParameter("Port_number"  ,m_portNumber,DEFAULT_PORT);

	//M\̂̏(mFwb_tb^̏)
	snd_data.head[0] = 'H';
	snd_data.head[1] = 'E';
	snd_data.head[2] = 'A';
	snd_data.head[3] = 'D';

	snd_data.tail[0] = 'T';
	snd_data.tail[1] = 'A';
	snd_data.tail[2] = 'I';
	snd_data.tail[3] = 'L';

  // </rtc-template>
  return RTC::RTC_OK;
}




RTC::ReturnCode_t VirtualH8::onFinalize()
{
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t VirtualH8::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VirtualH8::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t VirtualH8::onActivated(RTC::UniqueId ec_id)
{
	int flag(1);
	while(flag){
		try
		{
			if(m_requestID->getRequestFlag()==0){
				m_requestID->request(1,sizeof(receive_data_struct),"H8ComReceiveData");
				while(m_requestID->getRequestFlag()<2)usleep(1);
				rcv_shm_id = m_requestID->getShmID();
				rcv_sem_id = m_requestID->getSemID();
				m_requestID->setRequestFlag(5);
			}
			if(m_requestID->getRequestFlag()==5)
			{
				m_requestID->request(1,sizeof(send_data_struct),"H8ComSendData");
				while(m_requestID->getRequestFlag()<2)usleep(1);
				snd_shm_id = m_requestID->getShmID();
				snd_sem_id = m_requestID->getSemID();
				m_requestID->setRequestFlag(3);
			}
			flag = 0;
		}
		catch(...)
		{
			usleep(1000);
		}
	}
  return RTC::RTC_OK;
}



RTC::ReturnCode_t VirtualH8::onDeactivated(RTC::UniqueId ec_id)
{
	int flag(1);
	rcv_shm_id = -1;
	rcv_sem_id = -1;
	snd_shm_id = -1;
	snd_sem_id = -1;
	try{
		if(m_requestID->getRequestFlag()==0)flag=0;
	}
	catch(...)
	{

	}
	while(flag){
		try{
			if(m_requestID->getRequestFlag()==3)
			{
				m_requestID->request(-1,sizeof(receive_data_struct),"H8ComReceiveData");
				while(m_requestID->getRequestFlag()<2)usleep(1);
				m_requestID->setRequestFlag(5);
			}
			if(m_requestID->getRequestFlag()==5)
			{
				m_requestID->request(-1,sizeof(send_data_struct),"H8ComSendData");
				while(m_requestID->getRequestFlag()<2)usleep(1);
				m_requestID->setRequestFlag(0);
			}
			flag = 0;
		}
		catch(...)
		{

		}
	}
  return RTC::RTC_OK;
}



RTC::ReturnCode_t VirtualH8::onExecute(RTC::UniqueId ec_id)
{
	//MLXVꂽ
	if(m_informUpdateInPortIn.isNew())
	{
		m_informUpdateInPortIn.read();//ǂݎ̂
		rcvPtr = rcvClient.returnPointer(rcv_shm_id);
		if(rcvPtr != NULL)
		{
			rcvClient.lock(rcv_sem_id);
			memcpy(&rcv_data,rcvPtr,sizeof(receive_data_struct));
			rcvClient.unlock(rcv_sem_id);
		}

    //L̃|C^擾
		sndPtr = sndClient.returnPointer(snd_shm_id);
		if(sndPtr != NULL){

			//L̃f[^Rs[
			sndClient.lock(snd_sem_id);
			memcpy(&snd_data, sndPtr,sizeof(send_data_struct));
			sndClient.unlock(snd_sem_id);
			processCounter = DIVIDE_NUMBER_DEFAULT;
			for(int i(0);i<6;i++)
			{
				std::cout << "target"<< i << ": angle " << snd_data.angle[i] << " , speed "<<snd_data.speed[i] << "\n";
				angleAdder[i] = (double)(snd_data.angle[i]-rcv_data.angle[i])/(double)processCounter;
				speedAdder[i] = (double)(snd_data.speed[i]-rcv_data.speed[i])/(double)processCounter;
			}
			std::cout<<std::endl;
		}
		else std::cout << "Unable to get shared memory pointer (In sending)" << std::endl;
	}
	sleep(2);
	if(processCounter)
	{
    //ML̃|C^擾
		rcvPtr = rcvClient.returnPointer(rcv_shm_id);
		if(rcvPtr != NULL){
			rcvClient.lock(rcv_sem_id);
			rcvPtr->command = snd_data.command;
			if(processCounter==0)memcpy(rcvPtr,&snd_data,sizeof(send_data_struct));
			else for(int i(0);i<6;i++)
			{
				rcvPtr->angle[i] += (float)angleAdder[i];
				rcvPtr->speed[i] += (float)speedAdder[i];
				std::cout << "Now"<< i << ": angle " << rcvPtr->angle[i] << " , speed "<<rcvPtr->speed[i] << "\n";
			}
			std::cout<<std::endl;
			rcvClient.unlock(rcv_sem_id);

			//XVʒms
			m_informUpdateOutPort.explain = "H8ComReceiveData";
			m_informUpdateOutPort.byte = sizeof(receive_data_struct);
			m_informUpdateOutPortOut.write();
		}
		else std::cout << "Unable to get shared memory pointer (In receiving)" << std::endl;
		sleep(2);
		processCounter--;
	}
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t VirtualH8::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VirtualH8::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VirtualH8::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VirtualH8::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VirtualH8::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void VirtualH8Init(RTC::Manager* manager)
  {
    RTC::Properties profile(virtualh8_spec);
    manager->registerFactory(profile,
                             RTC::Create<VirtualH8>,
                             RTC::Delete<VirtualH8>);
  }
  
};


